/*
** Skinned.h
** 
** Made by damien
** Login   <damien@ortie>
** 
** Started on  Wed Feb  6 10:39:23 2008 damien
** Last update Wed Feb  6 10:39:23 2008 damien
*/

#ifndef   	SKINNED_H_
# define   	SKINNED_H_

#define WEIGHT_EXPONENT 4


#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <stdio.h>
using namespace std;


#include <Mesh.h>
#include <Skeleton.h>
#include <int_vector.h>

class Skinned
{
 public:

  Skinned();
  Skinned(const Skinned&);
  ~Skinned();




  Mesh *get_mesh();
  Mesh *get_compared_mesh();
  Mesh *get_deformed_mesh();
  Mesh *get_corrected_mesh();
  Skeleton *get_skeleton();
  Skeleton *get_deformed_skeleton();
  Skeleton *get_compared_skeleton();
  


  int set_skeleton(Skeleton *);
  int set_deformed_skeleton(Skeleton *_skeleton);
  int set_mesh(Mesh *);
  int set_deformed_mesh(Mesh *);


  //APPLY SKINNING
  int skinning();




  //weight
  int size_weight();
  int add_weight(double w);
  int set_weight(int k_index, double value);
  int set_weight(std::vector <double>& _weight);
  double get_weight(int k_index);//direct access from index
  double get_weight(int k_vertex,int k_index);//acces from
					      //index_weight
  int set_weight(int k_vertex,int k_index_bone,double value);


  //index weight
  int size_index_weight();
  int size_index_weight(int k_vertex);
  int set_size_index_weight(int size);
  int set_size_index_weight(int k_vertex,int size);
  int get_size_index_weight(int k_vertex);
  int add_index_weight(int k_vertex,int index);
  int add_index_weight(int k_vertex,std::vector <int>& index);
  int set_index_weight(int k_vertex,int k_index,int _index_weight);
  int set_index_weight(std::vector <int_vector>& _index_weight);
  int get_index_weight(int k_vertex,int k_index);

  //index joint
  int size_index_joint();
  int size_index_joint(int k_vertex);
  int set_size_index_joint(int size);
  int set_size_index_joint(int k_vertex,int size);
  int add_index_joint(int k_vertex,int index);
  int add_index_joint(int k_vertex,std::vector <int>& index);
  int set_index_joint(int k_vertex,int k_index,int _index_joint);
  int set_index_joint(std::vector <int_vector>& _index_joint);
  int get_index_joint(int k_vertex,int k_index);


  int fill_vertex_depending_on_bone();
  int get_number_of_vertex_depending_on_bone(int k_bone);
  int get_index_vertex_depending_on_bone(int k_bone,int k_index);







  int check_integrity_skinning();
  double get_min_distance_to_any_bone(int k_vertex);
  double get_vertex_to_bone_distance(int k_vertex,int k_bone);



  //segmentation
  int get_max_weight_influence(int k_vertex,int *k_bone_max,double *weight_max);
  

  //weight calculation
  int automatic_weight_calculation();
  double calculate_weight(int k_vertex,int k_joint);
  int normalize_weight();



 private:

  Mesh *mesh;
  Mesh *deformed_mesh;

  Skeleton *skeleton;
  Skeleton *deformed_skeleton;

  std::vector <double> weight;
  std::vector <int_vector> index_weight;
  std::vector <int_vector> index_joint;

  std::vector <int_vector> vertex_depending_on_bone;//vertex which
						    //depends on bone
						    //k
};


#endif    /* !SKINNED_H_ */
