
#include <OpenGL_drawer.h>

OpenGL_drawer::OpenGL_drawer():draw_list(),current_color()
{
  current_color.resize(3);
  draw_list.resize(0);
}

OpenGL_drawer::~OpenGL_drawer()
{
  unsigned int k=0;
  for(k=0;k<draw_list.size();k++)
    {
      if(glIsList(draw_list[k]))
	{
	  glDeleteLists(draw_list[k],1);

	  #ifdef DEBUG
	  printf("[draw_list %d deleted]\n",k);
	  #endif 
	}
    }

}

int OpenGL_drawer::init_opengl(int argc,char **argv,const Navigator_tool& navigator)
{
  //init window
  glutInit (&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowPosition(0,0);
  glutInitWindowSize(navigator.get_X_screen(),navigator.get_Y_screen());
  glutCreateWindow("win_1");


  //OpenGL set
  glClearColor(1.0f,1.0f,1.0f,1.0f);
  glPointSize(5);
  glLineWidth(2);

  glEnable(GL_COLOR_MATERIAL);
  //glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE );
  glEnable(GL_DEPTH_TEST);
  glDepthMask(GL_TRUE);


  glFrontFace(GL_CCW);//clockwise CW or CCW
  //glDisable(GL_CULL_FACE);
  glEnable(GL_CULL_FACE);

  glDisable(GL_LIGHTING);
  glDisable(GL_BLEND);
  glDisable(GL_TEXTURE_2D);
  glDisable(GL_FOG);



  //light
  float light_ambient[4]={0.4,0.4,0.4,1.0};
  float light_diffuse[4]={0.4,0.4,0.4,1.0};
  float light_specular[4]={1.0,1.0,1.0,1.0};
  float light_position [4] = {0.0,100.0,1000.0,1.0};

  float material_ambient [4] = {1.0,1.0,1.0,1.0};
  float material_diffuse [4] = {1.0,1.0,1.0,1.0};
  float material_specular [4] = {1.0,1.0,1.0,1.0};
  float material_exponent [1] = {60.0};


  glShadeModel(GL_SMOOTH);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material_ambient);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material_diffuse);  
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, material_specular);  
  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, material_exponent);

  return 0;
}





int OpenGL_drawer::draw_triangles(Mesh& mesh)
{
  int k_triangle=0;


  glColor4f(0.0,0.0,0.0,1.0);

  glDisable(GL_DEPTH_TEST);

  double lambda = 0.02;
  V_3D v1,v2,v3;
  V_3D n1,n2,n3;
  glBegin(GL_LINES);
  for(k_triangle=0;k_triangle<mesh.get_triangle_number();k_triangle++)
    {
      v1 = mesh.get_vertex_v3d(k_triangle,0);
      v2 = mesh.get_vertex_v3d(k_triangle,1);
      v3 = mesh.get_vertex_v3d(k_triangle,2);

      n1 = mesh.get_normal_v3d(k_triangle,0);
      n2 = mesh.get_normal_v3d(k_triangle,1);
      n3 = mesh.get_normal_v3d(k_triangle,2);

      v1 = v1+lambda*n1;
      v2 = v2+lambda*n2;
      v3 = v3+lambda*n3;

      glVertex3d(v1[0],v1[1],v1[2]);
      glVertex3d(v2[0],v2[1],v2[2]);

      glVertex3d(v2[0],v2[1],v2[2]);
      glVertex3d(v3[0],v3[1],v3[2]);

      glVertex3d(v3[0],v3[1],v3[2]);
      glVertex3d(v1[0],v1[1],v1[2]);
    }

  glEnd();
  glEnable(GL_DEPTH_TEST);

  return 0;
}

int OpenGL_drawer::draw_local_basis(Navigator_tool& navigator)
{
  
  glDisable(GL_LIGHTING);
  //glDisable(GL_DEPTH_TEST);

  glLoadIdentity();
  glTranslatef(0,0,navigator.get_dist_obs());
  glRotatef(navigator.get_theta_1(),1.0,0.0,0.0);
  glRotatef(navigator.get_theta_2(),0.0,1.0,0.0);

  glColor4f(1.0,0.0,0.0,1.0);
  glBegin(GL_LINES);
  glColor3f(1.0,0.0,0.0);
  glVertex3f(0.0,0.0,0.0);
  glVertex3f(0.01,0.0,0.0);

  glColor3f(0.0,1.0,0.0);
  glVertex3f(0.0,0.0,0.0);
  glVertex3f(0.0,0.01,0.0);

  glColor3f(0.0,0.0,1.0);
  glVertex3f(0.0,0.0,0.0);
  glVertex3f(0.0,0.0,0.01);
  glEnd();
  glTranslatef(0,0,navigator.get_dist_obs());

  //glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);

  return 0;
}






int OpenGL_drawer::draw_mesh(Mesh& mesh)
{

  glEnable(GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  //else draw fast
  const GLdouble *vertex = mesh.get_vertex();
  const GLdouble *normal = mesh.get_normal();
  const GLdouble *color = mesh.get_color();
  const GLint* index = mesh.get_index_vertex();
  GLint N_triangle = mesh.get_triangle_number();


  //glEnable(GL_TEXTURE_2D);
  glDisable(GL_TEXTURE_2D);

  //enable client
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_COLOR_ARRAY);

  //set pointer
  glVertexPointer(3,GL_DOUBLE,0,vertex);
  glNormalPointer(GL_DOUBLE,0,normal);
  glColorPointer(4,GL_DOUBLE,0,color);

  //transparency
  glEnable(GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);


  //draw
  glDrawElements(GL_TRIANGLES,N_triangle*3,GL_UNSIGNED_INT,index);

  //end transparency
  glDisable(GL_BLEND);

  //disable client
  glDisableClientState(GL_VERTEX_ARRAY);  
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_COLOR_ARRAY);


  glDisable(GL_BLEND);


  return 0;
}

int OpenGL_drawer::draw_skeleton(Skeleton &skeleton)
{
  int k_joint;
  V_3D p0,p1;

  Joint *current=NULL;
  Joint *son=NULL;

  int k_son=0;
  int N_son=0;

  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);

  glLineWidth(4.0);
  glBegin(GL_LINES);
  for(k_joint=0;k_joint<skeleton.get_N_joint()-1;k_joint++)
    {
      current = skeleton.get_joint(k_joint);
      p0 = skeleton.get_world_position_of_bone(current->get_access_number());

      N_son = current->get_N_son();

      set_current_color(current->get_access_number());
      glColor4d(current_color[0],current_color[1],current_color[2],1.0);
      for(k_son=0;k_son<N_son;k_son++)
	{
	  son = current->get_son(N_son-k_son-1);
	  p1 = skeleton.get_world_position_of_bone(son->get_access_number());

	  set_current_color(k_son);

	  glVertex3f(p0[0],p0[1],p0[2]);
	  glVertex3f(p1[0],p1[1],p1[2]);
	}
    }
  glEnd();

  glEnable(GL_LIGHTING);
  glEnable(GL_DEPTH_TEST);
	  
  return 0;
}

int OpenGL_drawer::set_current_color(int k_bone)
{

  double c1=abs(cos(double(k_bone*43252345.323+24)));
  double c2=abs(cos(double(k_bone*26252345.73+24)));
  double c3=abs(cos(double(k_bone*1214345.923+24)));
  current_color[0]=(c1+0.5)/2.0;
  current_color[1]=(c2+0.5)/2.0;
  current_color[2]=(c3+0.5)/2.0;

  return 0;
}

int OpenGL_drawer::draw_selected_bone(Skeleton &skeleton,int selected_joint)
{
  float radius_sphere=0.05;
  
  
  V_3D p;
  p = skeleton.get_world_position_of_bone(selected_joint);
  
  glDisable(GL_DEPTH_TEST);
  glLineWidth(3.0);
  glPushMatrix();
  glColor3f(1.0,0.0,0.0);
  glTranslatef(p[0],p[1],p[2]);
  glutWireSphere(radius_sphere,4,4);
  glPopMatrix();
  glLineWidth(1.0);
  glEnable(GL_DEPTH_TEST);


  return 0;
}

